// Sample code for the paper:
// "Exact simulation of integrate-and-fire models
// with exponential currents", R. Brette (2006)
//
// Simulation of a randomly connected network of IF models
// with exponential synaptic currents.
// Method: clock-driven simulation with exact subthreshold integration
// 
// The default behavior of the program is to output the spikes to
// the text file "spikes.dat". Uncomment the line "#define trace"
// if you just want to measure the simulation time.
//
// This program runs for about 5 min on a standard desktop PC.
// (reduce the parameter "duration" if this is too long)
//
// Sep 2006, R. Brette
// brette@di.ens.fr

#include <cstdlib>
#include <ctime>
#include <cmath>
#include <fstream>
#include <iostream>
using namespace std;

#define Ne		250		// number of excitatory neurons
#define Ni		150		// number of inhibitory neurons
#define we		7.		// weight of excitatory connections (mV)
#define wi		(-10.5)	// weight of inhibitory connections
#define Vr		0.		// reset potential (mV)
#define Vt		15.		// threshold potential
#define taum	20.		// membrane time constant (ms)
#define taue	5.		// excitatory synaptic time constant
#define taui	10.		// inhibitory synaptic time constant
#define cpr		0.2		// connection probability
#define dt		.1		// simulation time step
#define extrate	500.	// external Poisson rate (Hz)
#define weext	15.		// external excitatory weight
#define wiext	-5.		// external inhibitory weight
#define duration 600000.	// duration of the simulation (ms)

#define rnd()	(rand()*1./(RAND_MAX+1.))	// random number in [0..1]

// Comment the next line in order to refrain the program from outputing the results
// (e.g. in order to measure simulation time)
#define trace

int main()
{
	double V[Ne+Ni];	// membrane potential
	double Ie[Ne+Ni];	// excitatory current
	double Ii[Ne+Ni];	// inhibitory current
	int *C[Ne+Ni];		// lists of target neurons
	int nC[Ne+Ni];		// number of target neurons
	int k;

	//srand((unsigned)time(NULL));
	srand(1234);

	// Initialization (simulation clock starts after that)
	// (The number of targets for a given neuron follows a Bernouilli distribution
	// but we use a brutal method because it is easier to implement)
	int temp[Ne+Ni];
	for(int i=0;i<Ne+Ni;i++) {
		V[i]=Vr;
		Ie[i]=0;
		Ii[i]=0;
		nC[i]=0;
		for(int j=0;j<Ne+Ni;j++) {
			if (rnd()<cpr) {
				temp[j]=1;
				nC[i]++;
			} else
				temp[j]=0;
		}
		C[i]=new int[nC[i]]; // create the list
		k=0;
		for(int j=0;j<Ne+Ni;j++)
			if (temp[j])
				C[i][k++]=j;
	}

	// Initialization of the parameters for exact integration
	double ae=exp(-dt/taue);
	double ai=exp(-dt/taui);
	double bm=exp(-dt/taum);
	double be=(bm-ae)*taue/(taum-taue);
	double bi=(bm-ai)*taui/(taum-taui);
	double extp=extrate*0.001*dt; // probability of external spike in step

	cout << "Network initialized." << endl;

	clock_t t1=clock();

#ifdef trace
	ofstream file("spikes.dat");
#endif

	// Simulation
	for(double t=0.;t<duration;t+=dt) {
		// Integration
		for(int i=0;i<Ne+Ni;i++) {
			// Exact (linear) integration
			V[i]=V[i]*bm+Ie[i]*be+Ii[i]*bi;
			Ie[i]=Ie[i]*ae;
			Ii[i]=Ii[i]*ai;
			if (extp>=rnd())
				Ie[i]+=weext;
			if (extp>=rnd())
				Ii[i]+=wiext;
		}

		// Communication
		for(int i=0;i<Ne+Ni;i++) {
			// Spike?
			if (V[i]>=Vt) {
			#ifdef trace
				file << i << "\t" << t << endl;
			#endif
				V[i]=Vr;
				if (i<Ne)	// excitatory neuron
					for(int j=0;j<nC[i];j++)
						Ie[C[i][j]]+=we;
				else		// inhibitory neuron
					for(int j=0;j<nC[i];j++)
						Ii[C[i][j]]+=wi;
			}
		}
	}

	for(int i=0;i<Ne+Ni;i++)
		delete [] C[i];

#ifdef trace
	file.close();
#endif

	clock_t t2=clock();
	cout << "Time = " << (t2-t1)/CLOCKS_PER_SEC << " s" << endl;
	return 0;
}
